@@ -12,6 +12,7 @@ from trytond.report import Report
 from trytond.pyson import Eval, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
+from trytond.modules.company import CompanyReport
 
 
 __all__ = ['AuxiliaryBookStart', 'PrintAuxiliaryBook', 'AuxiliaryBook',
@@ -21,6 +22,8 @@ __all__ = ['AuxiliaryBookStart', 'PrintAuxiliaryBook', 'AuxiliaryBook',
         'PrintTrialBalanceStart', 'PrintTrialBalance', 'TrialBalance',
         'BalanceSheet', 'IncomeStatement', 'OpenCashflowStatement', 'Cashflow',
         'CashflowTemplate', 'OpenCashflowStart', 'Account',
+        'PrintPartyWithholding', 'PartyWithholdingStart',
+        'PartyWithholding',
         ]
 
 __metaclass__ = PoolMeta
@@ -1266,3 +1269,127 @@ class OpenCashflowStatement(Wizard):
                 'company': self.start.company.id,
                 })
         return action, {}
+
+
+class PartyWithholdingStart(ModelView):
+    'Party Withholding Start'
+    __name__ = 'account.party_withholding.start'
+    fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscalyear', 
+        required=True)
+    company = fields.Many2One('company.company', 'Company', required=True)
+    party = fields.Many2One('party.party', 'Party')
+    certificate_report = fields.Selection('selection_certificate_report',
+        'Certificate Report', required=True)
+    start_period = fields.Many2One('account.period', 'Start Period',
+        domain=[
+            ('fiscalyear', '=', Eval('fiscalyear')),
+            ('start_date', '<=', (Eval('end_period'), 'start_date')),
+            ], depends=['fiscalyear', 'end_period'])
+    end_period = fields.Many2One('account.period', 'End Period',
+        domain=[
+            ('fiscalyear', '=', Eval('fiscalyear')),
+            ('start_date', '>=', (Eval('start_period'), 'start_date'))
+            ],
+        depends=['fiscalyear', 'start_period'])
+
+    @staticmethod
+    def default_company():
+        return Transaction().context.get('company')
+
+
+    @classmethod
+    def selection_certificate_report(cls):
+        Tax = Pool().get('account.tax')
+        sel = Tax.certificate_report.selection
+        return sel
+
+
+class PrintPartyWithholding(Wizard):
+    'Print Withholding'
+    __name__ = 'account.print_party_withholding'
+    start = StateView('account.party_withholding.start',
+        'account_plus.print_party_withholding_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Ok', 'print_', 'tryton-ok', default=True),
+            ])
+    print_ = StateAction('account_plus.report_party_withholding')
+
+    def do_print_(self, action):
+        if self.start.start_period:
+            start_period = self.start.start_period.id
+        else:
+            start_period = None
+        if self.start.end_period:
+            end_period = self.start.end_period.id
+        else:
+            end_period = None
+
+        party_id = None
+        if self.start.party:
+            party_id = self.start.party.id
+        data = {
+            'company': self.start.company.id,
+            'fiscalyear': self.start.fiscalyear.id,
+            'party': party_id,
+            'certificate_report': self.start.certificate_report,
+            'start_period': start_period,
+            'end_period': end_period,
+            }
+        return action, data
+
+
+class PartyWithholding(CompanyReport):
+    __name__ = 'account.party_withholding'
+
+    @classmethod
+    def parse(cls, report, objects, data, localcontext):
+        pool = Pool()
+        InvoiceTax = pool.get('account.invoice.tax')
+        Tax = pool.get('account.tax')
+        Company = pool.get('company.company')
+        Period = pool.get('account.period')
+        Fiscalyear = pool.get('account.fiscalyear')
+        Party = pool.get('party.party')
+
+        company = Company(data['company'])
+        parties_ids = []
+        if data['party']:
+            parties_ids.append(data['party'])
+            party_dom = [
+                ('id', 'in', parties_ids),
+            ]
+        else:
+            party_dom = []
+        parties = Party.search(party_dom)
+
+        periods = Period.search_read([
+                ('fiscalyear', '=', data['fiscalyear']),
+            ], fields_names=['id'])
+        period_ids = [p.values()[0] for p in periods]
+
+        taxes = Tax.search_read([
+            ('certificate_report', '=', data['certificate_report'])
+            ], fields_names=['id'])
+        tax_ids = [t.values()[0] for t in taxes]
+
+        objects = []
+        for party in parties:
+            invoice_taxes = InvoiceTax.search([
+                    ('invoice.move.period', 'in', period_ids),
+                    ('invoice.party', '=', party.id),
+                    ('invoice.state', '=', 'posted'),
+                    ('tax', 'in', tax_ids),
+                    ], order=[('create_date', 'ASC')])
+            if not invoice_taxes:
+                continue
+            total_amount = sum([abs(i.amount) for i in invoice_taxes])
+            setattr(party, 'taxes', invoice_taxes)
+            setattr(party, 'total_amount', total_amount)
+            objects.append(party)
+
+        localcontext['fiscalyear'] = Fiscalyear(data['fiscalyear'])
+        localcontext['today'] = date.today()
+        localcontext['company'] = company
+        localcontext['reporte'] = data['certificate_report']
+        return super(PartyWithholding, cls).parse(report, objects, data,
+                localcontext)
